MODULE mo_kind

  ! The module mo_kind provides 'kind' parameters to specify
  ! the precision and range of real and integer operations in
  ! fortran 90:
  !
  !   - Kind parameter rs
  !     ---------------==
  !     REAL(rs) is the smallest REAL type of specified
  !     decimal precision provided by the fortran compiler.
  !
  !   - Kind parameter is
  !     ---------------==
  !   - INTEGER(is) is the smallest INTEGER type of specified
  !     range provided by the fortran compiler.
  !
  ! Author: Marco Giorgetta, Max Planck Institute for Meteorology
  !         August 2003

  IMPLICIT NONE

  PRIVATE

  ! Default kind values for REAL and INTEGER
  ! ========================================
  INTEGER,PARAMETER,PUBLIC :: rd =KIND(1.)
  INTEGER,PARAMETER,PUBLIC :: id =KIND(1)


  ! List of REALs selected by precision
  ! -----------------------------------
  !
  ! rn=SELECTED_REAL_KIND(n)
  !
  ! -> REAL(rn) allows a decimal precision of at least n digits
  !
  INTEGER,PARAMETER :: r5 =SELECTED_REAL_KIND(5)  ! maps to  4 byte REAL
  INTEGER,PARAMETER :: r10=SELECTED_REAL_KIND(10) ! maps to  8 byte REAL
  INTEGER,PARAMETER :: r20=SELECTED_REAL_KIND(20) ! maps to 16 byte REAL


  ! List of INTEGERs selected by range
  ! ----------------------------------
  !
  ! in=SELECTED_INT_KIND(n)
  !
  ! -> INTEGER(in) allows all integers within -10^n,...,10^n
  !
  ! n=2,4,9, or 18 are the maximum exponents such that
  ! the range -10^n,...,10^n is representable by 1,2,4, or 8 bytes.
  !
  INTEGER,PARAMETER :: i2 =SELECTED_INT_KIND(2)   ! maps to 1 byte integer
  INTEGER,PARAMETER :: i4 =SELECTED_INT_KIND(4)   ! maps to 2 byte integer
  INTEGER,PARAMETER :: i9 =SELECTED_INT_KIND(9)   ! maps to 4 byte integer
  INTEGER,PARAMETER :: i18=SELECTED_INT_KIND(18)  ! maps to 8 byte integer


  ! Select a REAL precison and a INTEGER range
  ! to define kind parameters rs and is
  ! ==========================================
  INTEGER,PARAMETER,PUBLIC :: rs=r10
  INTEGER,PARAMETER,PUBLIC :: is=i9

END MODULE mo_kind
